@@ -34,14 +34,14 @@ SOFTWARE.
 #include <saslplug.h>
 #include "saslint.h"
 #ifdef WIN32
-//for snprintf
+/* for snprintf */
 #include <stdio.h>
-//need to handle the fact that errno has been defined as a function
-//in a dll, not an extern int
+/* need to handle the fact that errno has been defined as a function
+   in a dll, not an extern int */
 #ifdef errno
 #undef errno
-#endif //errno
-#endif //WIN32
+#endif /* errno */
+#endif /* WIN32 */
 
 int _sasl_debug = 0;
 
@@ -544,7 +544,7 @@ static int _sasl_syslog(void *context __attribute__((unused)),
 
 #ifdef WIN32
 #define MAX_VALUE_NAME	128
-#endif //WIN32
+#endif /* WIN32 */
 
 static int
 _sasl_getsimple(void *context,
@@ -571,24 +571,24 @@ _sasl_getsimple(void *context,
     userid = getenv("USER");
 #else
     userid = getenv("USERNAME");
-#endif //WIN32
+#endif /* WIN32 */
     if (userid != NULL) {
       *result = userid;
       if (len) *len = strlen(userid);
       return SASL_OK;
     }
 #ifdef WIN32
-    //for win32, try using the GetUserName standard call
+    /* for win32, try using the GetUserName standard call */
     else {
 	i=sizeof(sender);
 	rval = GetUserName(sender, &i);
-	if ( rval) { //got a userid
+	if ( rval) { /* got a userid */
 		*result = sender;
 		if (len) *len = strlen(sender);
 		return SASL_OK;
 	}
     }
-#endif //WIN32
+#endif /* WIN32 */
     return SASL_FAIL;
   default:
     return SASL_BADPARAM;
@@ -643,7 +643,7 @@ _sasl_getcallback(sasl_conn_t * conn,
 {
   const sasl_callback_t *callback;
 
-  if (! conn || ! pproc || ! pcontext)
+  if (! pproc || ! pcontext)
     return SASL_BADPARAM;
 
   /* Some callbacks are always provided by the library */
@@ -659,33 +659,38 @@ _sasl_getcallback(sasl_conn_t * conn,
 
   /* If it's not always provided by the library, see if there's
    * a version provided by the application for this connection... */
-  if (conn->callbacks)
-    for (callback = conn->callbacks;
-	 callback->id != SASL_CB_LIST_END;
-	 callback++)
-      if (callback->id == callbackid) {
-	*pproc = callback->proc;
-	*pcontext = callback->context;
-	if (callback->proc)
-	  return SASL_OK;
-	else
-	  return SASL_INTERACT;
-      }
+  if (conn && conn->callbacks) {
+    for (callback = conn->callbacks; callback->id != SASL_CB_LIST_END;
+	 callback++) {
+	if (callback->id == callbackid) {
+	    *pproc = callback->proc;
+	    *pcontext = callback->context;
+	    if (callback->proc) {
+		return SASL_OK;
+	    } else {
+		return SASL_INTERACT;
+	    }
+	}
+    }
+  }
 
   /* And, if not for this connection, see if there's one
    * for all {server,client} connections... */
-  if (conn->global_callbacks && conn->global_callbacks->callbacks)
-    for (callback = conn->global_callbacks->callbacks;
-	 callback->id != SASL_CB_LIST_END;
-	 callback++)
-      if (callback->id == callbackid) {
-	*pproc = callback->proc;
-	*pcontext = callback->context;
-	if (callback->proc)
-	  return SASL_OK;
-	else
-	  return SASL_INTERACT;
+  if (conn && conn->global_callbacks && conn->global_callbacks->callbacks) {
+      for (callback = conn->global_callbacks->callbacks;
+	   callback->id != SASL_CB_LIST_END;
+	   callback++) {
+	  if (callback->id == callbackid) {
+	      *pproc = callback->proc;
+	      *pcontext = callback->context;
+	      if (callback->proc) {
+		  return SASL_OK;
+	      } else {
+		  return SASL_INTERACT;
+	      }
+	  }
       }
+  }
 
   /* Otherwise, see if the library provides a default callback. */
   switch (callbackid) {
