@@ -1,6 +1,6 @@
 /* SASL server API implementation
  * Tim Martin
- * $Id: server.c,v 1.46 1999/08/13 16:19:52 leg Exp $
+ * $Id: server.c,v 1.54 1999/10/10 17:38:45 leg Exp $
  */
 /***********************************************************
         Copyright 1998 by Carnegie Mellon University
@@ -119,12 +119,21 @@ external_server_step(void *conn_context __attribute__((unused)),
       || !sparams->utils->getcallback
       || !serverout
       || !serveroutlen
-      || !oparams
-      || !errstr)
+      || !oparams)
     return SASL_BADPARAM;
 
+  if (errstr) {
+      *errstr = NULL;
+  }
+
   if (!sparams->utils->conn->external.auth_id)
     return SASL_BADPROT;
+
+  if ((sparams->props.security_flags & SASL_SEC_NOANONYMOUS) &&
+      (!strcmp(sparams->utils->conn->external.auth_id, "anonymous"))) {
+      *errstr = "anonymous login not allowed";
+      return SASL_NOAUTHZ;
+  }
   
   if (! clientin) {
     /* No initial data; we're in a protocol which doesn't support it.
@@ -168,7 +177,6 @@ external_server_step(void *conn_context __attribute__((unused)),
   oparams->verifymic = NULL;
   oparams->realm = NULL;
   oparams->param_version = 0;
-  *errstr = NULL;
 
   return SASL_OK;
 }
@@ -177,6 +185,7 @@ static const sasl_server_plug_t external_server_mech = {
   "EXTERNAL",			/* mech_name */
   0,				/* max_ssf */
   SASL_SEC_NOPLAINTEXT
+  | SASL_SEC_NOANONYMOUS
   | SASL_SEC_NODICTIONARY,	/* security_flags */
   NULL,				/* glob_context */
   &external_server_new,		/* mech_new */
@@ -235,16 +244,6 @@ typedef struct sasl_server_conn {
 
   int authenticated;
   mechanism_t *mech; /* mechanism trying to use */
-  /* data for mechanism to use so can "remember" challenge thing
-   * for example kerberos sends a random integer
-   */ 
-  union mech_data  
-  {
-    int Idata;  
-    double Fdata;  
-    char *Sdata;  
-  } mech_data_t;
-
   sasl_server_params_t *sparams;
 
 } sasl_server_conn_t;
@@ -286,55 +285,39 @@ int sasl_setpass(sasl_conn_t *conn,
     void *context;
     const char *plainmech;
 
-    /* Zowie -- we have the user's plaintext password.
-     * Let's tell all our mechanisms about it...
-     */
-
-    if (!conn || !pass || !mechlist)
+    if (!conn)
+	return SASL_BADPARAM;
+    
+    if (!mechlist) {
 	return SASL_FAIL;
+    }
 
-    VL(("Setting password for \"%s\" to \"%*s\" (len is %d)\n",
-	user, passlen, pass, passlen));
+    if ((flags & SASL_SET_CREATE) && (flags & SASL_SET_DISABLE)) {
+	return SASL_BADPARAM;
+    }
 
-    _sasl_log(conn, SASL_LOG_INFO, NULL, 0, 0,
-	      "updating secrets for %s", user);
+    if (errstr) { *errstr = NULL; }
 
     if (_sasl_getcallback(conn, SASL_CB_GETOPT, 
 			  &getopt, &context) == SASL_OK) {
 	getopt(context, NULL, "pwcheck_method", &plainmech, NULL);
     }
 
-    if (plainmech != NULL && !strcmp(plainmech, "sasldb")) {
-	/* we store the user's password for plaintext checking */
-	sasl_secret_t *sec = NULL;
-	char salt[16];
-	sasl_rand_t *rpool;
-	int ret;
-	sasl_server_putsecret_t *putsec;
-	void *context;
-
-	ret = sasl_randcreate(&rpool);
-	if (ret == SASL_OK) {
-	    sasl_rand(rpool, salt, 16);
-	    ret = _sasl_make_plain_secret(salt, pass, passlen, &sec);
-	}
-	if (ret == SASL_OK) {
-	    ret = _sasl_getcallback(conn, SASL_CB_SERVER_PUTSECRET, &putsec,
-				    &context);
-	}
-	if (ret == SASL_OK) {
-	    ret = putsec(context, "PLAIN", user, sec);
-	}
-	if (ret != SASL_OK) {
-	    _sasl_log(conn, SASL_LOG_ERR, NULL, ret, 0,
-		      "failed to set plaintext secret for %s: %z", user);
-	}
-	if (rpool) {
-	    sasl_randfree(&rpool);
-	}
-	if (sec) {
-	    sasl_free_secret(&sec);
-	}
+    /* set/create password for PLAIN usage */
+    tmpresult = _sasl_sasldb_set_pass(conn, user, pass, passlen, 
+				      s_conn->user_realm, flags, errstr);
+    if (tmpresult != SASL_OK && tmpresult != SASL_NOCHANGE) {
+	result = SASL_FAIL;
+	_sasl_log(conn, SASL_LOG_ERR, "PLAIN", tmpresult,
+#ifndef WIN32
+		  errno,
+#else
+		  GetLastError(),
+#endif
+		  "failed to set secret for %s: %z", user);
+    } else {
+	_sasl_log(conn, SASL_LOG_INFO, "PLAIN", 0, 0, 
+		  "set secret for %s", user);
     }
 
     /* copy info into sparams */
@@ -344,34 +327,32 @@ int sasl_setpass(sasl_conn_t *conn,
 
     /* now we let the mechanisms set their secrets */
     for (m = mechlist->mech_list; m; m = m->next) {
-	if (m->plug->setpass) {
-	    VL(("Setting it for mech %s\n", m->plug->mech_name));
-	    tmpresult = m->plug->setpass(m->plug->glob_context,
-					 ((sasl_server_conn_t *)conn)->sparams,
-					 user,
-					 pass,
-					 passlen,
-					 flags,
-					 errstr);
-	    if (tmpresult != SASL_OK) {
-		VL(("%s returned %i\n",m->plug->mech_name, tmpresult));
-		result = SASL_FAIL;
-		_sasl_log(conn, SASL_LOG_ERR, m->plug->mech_name, tmpresult,
+	if (!m->plug->setpass) {
+	    /* can't set pass for this mech */
+	    continue;
+	}
+	tmpresult = m->plug->setpass(m->plug->glob_context,
+				     ((sasl_server_conn_t *)conn)->sparams,
+				     user,
+				     pass,
+				     passlen,
+				     flags,
+				     errstr);
+	if (tmpresult == SASL_OK) {
+	    _sasl_log(conn, SASL_LOG_INFO, m->plug->mech_name, 0, 0,
+		      "set secret for %s", user);
+	} else if (tmpresult == SASL_NOCHANGE) {
+	    _sasl_log(conn, SASL_LOG_INFO, m->plug->mech_name, 0, 0,
+		      "secret not changed for %s", user);
+	} else {
+	    result = SASL_FAIL;
+	    _sasl_log(conn, SASL_LOG_ERR, m->plug->mech_name, tmpresult,
 #ifndef WIN32
-			  errno,
+		      errno,
 #else
-			  GetLastError(),
+		      GetLastError(),
 #endif
-			  "failed to set secret for %s: %z", user);
-	    } else {
-		VL(("%s succeeded!\n",m->plug->mech_name));
-		_sasl_log(conn,
-			  SASL_LOG_INFO,
-			  m->plug->mech_name,
-			  0, 0,
-			  "set secret for %s",
-			  user);
-	    }
+		      "failed to set secret for %s: %z", user);
 	}
     }
 
@@ -736,9 +717,9 @@ int sasl_server_new(const char *service,
   serverconn->sparams->props = serverconn->base.props;
 
   /* set some variables */
-  if (user_realm==NULL)
-    serverconn->user_realm=NULL;
-  else {
+  if (user_realm==NULL) {
+    serverconn->user_realm = NULL;
+  } else {
     result = _sasl_strdup(user_realm, &serverconn->user_realm, NULL);
   }
 
@@ -1057,38 +1038,55 @@ int sasl_listmech(sasl_conn_t *conn,
   
 }
 
+#define EOSTR(s,n) (((s)[n] == '\0') || ((s)[n] == ' ') || ((s)[n] == '\t'))
+static int is_mech(const char *t, const char *m)
+{
+    int sl = strlen(m);
+    return ((!strncasecmp(m, t, sl)) && EOSTR(t, sl));
+}
+
 /* returns OK if it's valid */
 static int _sasl_checkpass(sasl_conn_t *conn,
 			   const char *mech, const char *service,
 			   const char *user, const char *pass)
 {
+    sasl_server_conn_t *s_conn = (sasl_server_conn_t *) conn;
     int result = SASL_NOMECH;
 
 #ifdef HAVE_PAM
-    if (!strcmp(mech, "PAM")) {
+    if (is_mech(mech, "PAM")) {
 	result = _sasl_PAM_verify_password(conn, user, pass, service, NULL);
     } else
 #endif
 #ifndef SASL_MINIMAL_SERVER
-    if (!strcmp(mech, "passwd")) {
+    if (is_mech(mech, "passwd")) {
 	result = _sasl_passwd_verify_password(conn, user, pass, NULL);
     } else
-#endif //SASL_MINIMAL_SERVER
-    if (!strcmp(mech, "shadow")) {
+    if (is_mech(mech, "shadow")) {
 	result = _sasl_shadow_verify_password(conn, user, pass, NULL);
     } else
-
+#endif /* SASL_MINIMAL_SERVER */
 #ifdef HAVE_KRB
-    if (!strcmp(mech, "kerberos_v4")) {
+    if (is_mech(mech, "kerberos_v4")) {
 	/* check against krb */
 	result = _sasl_kerberos_verify_password(conn, user, pass, 
 						service, NULL);
     } else
 #endif
-
-    if (!strcmp(mech, "sasldb")) {
+#ifdef HAVE_PWCHECK
+    if (is_mech(mech, "pwcheck")) {
+	/* check against pwcheck daemon */
+	result = _sasl_pwcheck_verify_password(conn, user, pass, NULL);
+    } else
+#endif
+    if (is_mech(mech, "sasldb")) {
 	/* check sasl database */
-	result = _sasl_sasldb_verify_password(conn, user, pass, NULL);
+	result = _sasl_sasldb_verify_password(conn, user, pass, 
+					      s_conn->user_realm, NULL);
+    } else {
+	/* no mechanism available ?!? */
+	_sasl_log(conn, SASL_LOG_ERR, NULL, 0, 0,
+		  "unrecognized plaintext verifier %s", mech);
     }
 
     return result;
